#!/bin/sh
# the next line restarts using wish \
exec wish "$0" "$@";

# DirList --
#    This script is a Tk-based directory listing utility

# Create frames for the various sections
frame .control_frame1;
frame .control_frame2;
frame .text_frame;
frame .status_frame;
grid .control_frame1 -column 0 -row 0;
grid .text_frame -column 0 -row 1;
grid .control_frame2 -column 0 -row 2;
grid .status_frame -column 0 -row 3;

# Variables and key bindings
set DirListVersion "1.0.1";
set dirSelection "";
bind . <F1> aboutBox;
bind . <Control-q> exit;
bind . <Control-Q> exit;

# make_controls --
#    Make the controls that appears in the 2 control frames
proc make_controls {} {
  global dirSelection;

  label .control_frame1.dirLabel -text "Directory:" -width 15;
  entry .control_frame1.entry -textvariable dirSelection -width 40;
  button .control_frame1.choose -text "Choose..." -command chooseDir -width 15;
  pack .control_frame1.dirLabel .control_frame1.entry .control_frame1.choose -in .control_frame1 -side left -padx 5 -pady 5;

  button .control_frame2.run -text "List" -command listDir -width 15;
  button .control_frame2.copy -text "Copy" -command copyText -width 15;
  button .control_frame2.about -text "About" -command aboutBox -width 15;
  button .control_frame2.quit -text "Quit" -command exit -width 15;
  pack .control_frame2.run .control_frame2.copy .control_frame2.about .control_frame2.quit -in .control_frame2 -side left -padx 5 -pady 5;
}

# make_status_label --
#    Make the status bar at the bottom of the window
#
proc make_status_label {} {
  label .status_label -relief sunken -width 75;
  pack .status_label -anchor s -in .status_frame \
    -padx 2 -pady 2 -fill both -expand true;
}

# make_text_area --
#    - Make the area that the text from the file appears in
#
proc make_text_area {} {
  text .file_text -width 75 -height 15 -setgrid true -takefocus true \
    -borderwidth 2 -exportselection true -yscrollcommand {.yscroll set} \
    -relief sunken -highlightcolor gray90;
  scrollbar .yscroll -command {.file_text yview};
  pack .file_text -in .text_frame -side left -fill both -expand true;
  pack .yscroll -in .text_frame -side right -fill y;
  .file_text insert end "";
}

# chooseDir --
#    Choose the directory to list
proc chooseDir {} {
  global dirSelection;

  set dirSelection [tk_chooseDirectory -parent . -title "Choose a directory..." -mustexist true];
  .status_label configure -text "Select List to show the directory contents";
}

# listDir --
#    List the files in the selected directory
proc listDir {} {
  global dirSelection;

  set dirExists [file exists $dirSelection];
  set dirIsDir [file isdirectory $dirSelection];
  .status_label configure -text "Please choose a directory";
  if {$dirSelection eq ""} then {
    tk_messageBox -icon warning -type ok -title "Choose A Directory" -message \
      "Please choose a directory a first!";
  } elseif {$dirExists !=1 || $dirIsDir !=1} then {
    tk_messageBox -icon warning -type ok -title "Unknown Directory" -message \
      "Unable to locate selected directory!\n\nPlease choose a new directory.";
  } elseif {[catch {glob -directory $dirSelection -tails -type f *} currDirList]} then {
    tk_messageBox -icon warning -type ok -title "No Files In Directory" -message \
      "The selected directory does not contain any files!";
  } else {
    set currDirList [lsort -dictionary $currDirList];
    .file_text delete 1.0 end;
    for {set x 0} {$x < [llength $currDirList]} {incr x} {
      .file_text insert end [lindex $currDirList $x];
      .file_text insert end "\n";
    }
    .status_label configure -text "Select Copy to copy the directory list";
  }
}

# copyText --
#    Copy the selected text
#
proc copyText {} {
  .file_text tag add sel 1.0 end;
  tk_textCopy .file_text;
  .file_text tag remove sel 1.0 end;
  .status_label configure -text "The displayed list has been copied to the clipboard.";
}

# aboutBox --
#    Pops up a message box with an "about" message
#
proc aboutBox {} {
  global DirListVersion;
  tk_messageBox -icon info -type ok -title "About DirList" -message \
    "Dir List\n    A simple directory listing utility written in Tcl/Tk.\n\nVersion $DirListVersion\n\nCopyright (c) 2007\n\tMichael Talbot";
}

#
# Run the program
#
wm title . "Directory List Utility";
wm geometry . +30+30;
make_text_area;
make_controls;
make_status_label;
.status_label configure -text "Please choose a directory";
wm resizable . 0 0
